@@ -6,10 +6,13 @@ from keras_cv_attention_models.common_layers import (
     conv2d_no_bias,
     depthwise_conv2d_no_bias,
     drop_block,
+    eca_module,
     hard_swish,
     layer_norm,
     make_divisible,
     se_module,
+    tpu_extract_patches_overlap_1,
+    tpu_compatible_extract_patches_overlap_1,
 )
 from keras_cv_attention_models.aotnet.aotnet import aot_stem
 from keras_cv_attention_models.botnet.botnet import RelativePositionalEmbedding, mhsa_with_relative_position_embedding
@@ -22,3 +25,4 @@ from keras_cv_attention_models.mlp_family.mlp_mixer import mlp_block, mixer_bloc
 from keras_cv_attention_models.mlp_family.res_mlp import ChannelAffine
 from keras_cv_attention_models.mlp_family.gated_mlp import spatial_gating_block
 from keras_cv_attention_models.levit.levit import MultiHeadPositionalEmbedding, mhsa_with_multi_head_position_and_strides
+from keras_cv_attention_models.nfnets.nfnets import ScaledStandardizedConv2D, ZeroInitGain
