@@ -22,12 +22,14 @@ def light_multi_head_self_attention(inputs, num_heads=4, key_dim=0, sr_ratio=1,
     # print(f">>>> {inputs.shape = }, {query.shape = }, {final_out_shape = }, {strides = }")
     _, query_hh, query_ww, _ = query.shape
     query = tf.reshape(query, [-1, query_hh, query_ww, num_heads, key_dim])
-    pos_query = tf.transpose(query, [0, 3, 1, 2, 4])    # [batch, num_heads, hh, ww, key_dim]
-    attn_query = tf.reshape(pos_query, [-1, num_heads, query_hh * query_ww, key_dim])   # [batch, num_heads, hh * ww, key_dim]
+    pos_query = tf.transpose(query, [0, 3, 1, 2, 4])  # [batch, num_heads, hh, ww, key_dim]
+    attn_query = tf.reshape(pos_query, [-1, num_heads, query_hh * query_ww, key_dim])  # [batch, num_heads, hh * ww, key_dim]
 
     if sr_ratio > 1:
-        key_value = conv2d_no_bias(inputs, cc, kernel_size=sr_ratio, strides=sr_ratio, name=name + "kv_sr_")
+        key_value = depthwise_conv2d_no_bias(inputs, kernel_size=sr_ratio, strides=sr_ratio, name=name + "kv_sr_")
+        # key_value = conv2d_no_bias(inputs, inputs.shape[-1], kernel_size=sr_ratio, strides=sr_ratio, name=name + "kv_sr_")
         key_value = layer_norm(key_value, name=name+"kv_sr_")
+        # key_value = inputs[:, ::sr_ratio, ::sr_ratio, :]
     else:
         key_value = inputs
 
@@ -37,7 +39,7 @@ def light_multi_head_self_attention(inputs, num_heads=4, key_dim=0, sr_ratio=1,
     # key_value = tf.reshape(qkv, [-1, hh * ww, 2, num_heads, key_dim])
     # key, value = tf.transpose(key_value, [2, 0, 3, 1, 4])   # [2, batch, num_heads, blocks, key_dim]
     key_value = tf.reshape(key_value, [-1, key_hh * key_ww, key_dim, num_heads, 2])
-    key, value = tf.transpose(key_value, [4, 0, 3, 1, 2])   # [2, batch, num_heads, blocks, key_dim]
+    key, value = tf.transpose(key_value, [4, 0, 3, 1, 2])  # [2, batch, num_heads, blocks, key_dim]
 
     # scaled_dot_product_attention
     # print(f">>>> {attn_query.shape = }, {key.shape = }, {value.shape = }, {kv_inp.shape = }, {pos_query.shape = }")
@@ -67,29 +69,36 @@ def light_multi_head_self_attention(inputs, num_heads=4, key_dim=0, sr_ratio=1,
     # attention_output.set_shape(final_out_shape)
     return attention_output
 
+
 def inverted_residual_feed_forward(inputs, expansion=4, activation="gelu", name=""):
+    """ IRFFN(X) = Conv(F(Conv(X))), F(X) = DWConv(X) + X """
     in_channel = inputs.shape[-1]
-    expanded = conv2d_no_bias(inputs, int(in_channel * expansion), kernel_size=1, name=name + "1_")
+    expanded = conv2d_no_bias(inputs, int(in_channel * expansion), kernel_size=1, use_bias=True, name=name + "1_")
     expanded = batchnorm_with_activation(expanded, activation=activation, act_first=True, name=name + "1_")
 
-    dw = depthwise_conv2d_no_bias(expanded, kernel_size=3, padding="SAME", name=name)
-    dw = batchnorm_with_activation(dw, activation=activation, act_first=True, name=name + "2_")
+    dw = depthwise_conv2d_no_bias(expanded, kernel_size=3, padding="SAME", use_bias=True, name=name)
+    # dw = batchnorm_with_activation(dw, activation=activation, act_first=True, name=name + "2_")
     dw_out = keras.layers.Add(name=name + "dw_out")([expanded, dw])
+    dw_out = batchnorm_with_activation(dw_out, activation=activation, act_first=True, name=name + "2_")
 
-    pw = conv2d_no_bias(dw_out, in_channel, kernel_size=1, name=name + "3_")
+    pw = conv2d_no_bias(dw_out, in_channel, kernel_size=1, use_bias=False, name=name + "3_")
     pw = batchnorm_with_activation(pw, activation=None, name=name + "3_")
     return pw
 
+
 def cmt_block(inputs, num_heads=4, sr_ratio=1, expansion=4, activation="gelu", drop_rate=0, name=""):
-    """ local_perception_unit """
-    lpu = depthwise_conv2d_no_bias(inputs, kernel_size=3, padding="SAME", name=name)
+    """ X0 = LPU(Xi), X1 = LMHSA(LN(X0)) + X0, X2 = IRFFN(LN(X1)) + X1 """
+    """ Local Perception Unit, LPU(X) = DWConv(X) + X """
+    lpu = depthwise_conv2d_no_bias(inputs, kernel_size=3, padding="SAME", use_bias=True, name=name)
     # lpu = batchnorm_with_activation(lpu, activation=activation, name=name + "lpu_", act_first=True)
     lpu_out = keras.layers.Add(name=name + "lpu_out")([inputs, lpu])
 
+    """ light multi head self attention """
     attn = layer_norm(lpu_out, name=name + "attn_")
     attn = light_multi_head_self_attention(attn, num_heads=num_heads, sr_ratio=sr_ratio, name=name + "light_mhsa_")
     attn_out = keras.layers.Add(name=name + "attn_out")([lpu_out, attn])
 
+    """ inverted residual feed forward """
     ffn = layer_norm(attn_out, name=name + "ffn_")
     ffn = inverted_residual_feed_forward(ffn, expansion=expansion, activation=activation, name=name + "ffn_")
     ffn = drop_block(ffn, drop_rate=drop_rate)
@@ -97,15 +106,17 @@ def cmt_block(inputs, num_heads=4, sr_ratio=1, expansion=4, activation="gelu", d
 
     return ffn_out
 
+
 def cmt_stem(inputs, stem_width, activation="gelu", name="", **kwargs):
     nn = conv2d_no_bias(inputs, stem_width, kernel_size=3, strides=2, padding="same", name=name + "1_")
-    nn = batchnorm_with_activation(nn, activation=activation, act_first=True, name=name + "1_")
+    nn = batchnorm_with_activation(nn, activation=activation, act_first=False, name=name + "1_")
     nn = conv2d_no_bias(nn, stem_width, kernel_size=3, strides=1, padding="same", name=name + "2_")
-    nn = batchnorm_with_activation(nn, activation=activation, act_first=True, name=name + "2_")
+    nn = batchnorm_with_activation(nn, activation=activation, act_first=False, name=name + "2_")
     nn = conv2d_no_bias(nn, stem_width, kernel_size=3, strides=1, padding="same", name=name + "3_")
-    nn = batchnorm_with_activation(nn, activation=activation, act_first=True, name=name + "3_")
+    nn = batchnorm_with_activation(nn, activation=activation, act_first=False, name=name + "3_")
     return nn
 
+
 def CMT(
     num_blocks,
     out_channels,
@@ -130,18 +141,21 @@ def CMT(
     total_blocks = sum(num_blocks)
     global_block_id = 0
     for stack_id, (num_block, out_channel, num_head, sr_ratio) in enumerate(zip(num_blocks, out_channels, num_heads, sr_ratios)):
-        nn = conv2d_no_bias(nn, out_channel, kernel_size=2, strides=2, name="stage_{}_down_sample".format(stack_id + 1))
+        stage_name = "stage_{}_".format(stack_id + 1)
+        nn = conv2d_no_bias(nn, out_channel, kernel_size=2, strides=2, name=stage_name + "down_sample")
+        # nn = layer_norm(nn, name=stage_name)
         for block_id in range(num_block):
-            name = "stage_{}_block_{}_".format(stack_id + 1, block_id + 1)
-            strides = 2 if block_id == 0 else 1
-            conv_short_cut = True if block_id == 0 else False
+            name = stage_name + "block_{}_".format(block_id + 1)
             block_drop_rate = drop_connect_rate * global_block_id / total_blocks
             global_block_id += 1
             nn = cmt_block(nn, num_head, sr_ratio, ffn_expansion, activation=activation, drop_rate=block_drop_rate, name=name)
 
+    nn = conv2d_no_bias(nn, 1280, 1, strides=1, name="post_")
+    nn = batchnorm_with_activation(nn, activation=activation, name="post_")
+
     if num_classes > 0:
         nn = keras.layers.GlobalAveragePooling2D(name="avg_pool")(nn)
-        nn = keras.layers.Dense(1280, name="post_dense")(nn)
+        # nn = keras.layers.Dense(1280, name="post_dense")(nn)
         if drop_rate > 0:
             nn = keras.layers.Dropout(drop_rate)(nn)
         nn = keras.layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="predictions")(nn)
@@ -149,6 +163,7 @@ def CMT(
     model = keras.models.Model(inputs, nn, name=model_name)
     return model
 
+
 BLOCK_CONFIGS = {
     "tiny": {
         "num_blocks": [2, 2, 10, 2],
